package de.ipk.pdwexpress.parser.flarex;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import org.apache.commons.dbutils.DbUtils;
import org.apache.log4j.Logger;

import de.ipk.pdwexpress.peobjects.PE_ARRAYS;
import de.ipk.pdwexpress.peobjects.PE_BIOMATERIALS;
import de.ipk.pdwexpress.peobjects.PE_BIOSEQUENCES;
import de.ipk.pdwexpress.peobjects.PE_EXPERIMENTS;
import de.ipk.pdwexpress.peobjects.PE_ExternDBEnum;
import de.ipk.pdwexpress.peobjects.PE_HYBRIDISATIONS;
import de.ipk.pdwexpress.peobjects.PE_RESULTSETS;
import de.ipk.pdwexpress.tableObjects.EditingSession;
import de.ipk.pdwexpress.upload.AbstractPEObject;
import de.ipk.pdwexpress.upload.ExternDBInterface;
import de.ipk.pdwexpress.upload.ImportHandler;
import de.ipk.pdwexpress.upload.PE_Object;
import de.ipk.pdwexpress.util.PE_ObjectsWrapper;
import de.ipk.pdwexpress.util.PE_ObjectsWrapperImpl;
import de.ipk.pdwexpress.util.ResultDataHandler;

public class FlarexExperimentImport implements FLAREXQueries{
	
	
	public final static String IDENTIFIER_PREFIX="de.ipk-gatersleben.flarex:";
	/**
	 * es wird nur ein ResultSet importiert
	 */
	private final boolean testImport =false;	
	
	protected Logger logger;
			
	/**
	 * Connection to FLAREX
	 */
	protected final Connection conFLAREX;
	
	/**
	 * Connection PDW Express
	 */
	protected final Connection conPDWExpress;
	
	/**
	 * ID of FLAREX Experiment (FLAREX.FS_EXPERIMENTS.EXP_ID)
	 */
	protected final String exp_id;
	
	/**
	 * ID of the FLAREX Normalisation of the Experiment (FLAREX.FS_PIPELINE_APPLICATIONS.PAP_ID)
	 */
	protected final String pap_id;
	
	private final ImportHandler ih;
	


	protected PreparedStatement stmtExp;
	protected PreparedStatement stmtMat;
	protected PreparedStatement stmtHyb;
	protected PreparedStatement stmtRst;
	protected PreparedStatement stmtSeq;
	protected PreparedStatement stmtArr;
	protected PreparedStatement stmtHrs;
	protected PreparedStatement stmtNrs;
	protected PreparedStatement stmtErs;



	
	
	/**
	 * 
	 * @param conFLAREX Connection to FLAREX
	 * @param conPDWExpress Connection PDW Express
	 * @param exp_id  ID of FLAREX Experiment (FLAREX.FS_EXPERIMENTS.EXP_ID)
	 * @param pap_id ID of the FLAREX Normalisation of the Experiment (FLAREX.FS_PIPELINE_APPLICATIONS.PAP_ID)
	 * @throws SQLException If preparing of Statements or initiating of ImportHandler failed 
	 */
	public FlarexExperimentImport(Connection conFLAREX, Connection conPDWExpress, String exp_id, String pap_id) throws SQLException {
		
//		URL url = ClassLoader.getSystemResource("log4j.properties");
//		System.out.println(url);
//		PropertyConfigurator.configure(url);
		
		
		this.logger=Logger.getLogger(this.getClass());
	
		this.conFLAREX = conFLAREX;
		this.conPDWExpress = conPDWExpress;
		this.exp_id=exp_id;
		this.pap_id=pap_id;
		
		EditingSession ese = new EditingSession("Import of FLAREX data for exp_id:"+exp_id+" pap_id:"+pap_id);
		ih = new ImportHandler(conPDWExpress,ese);
		AbstractPEObject.setImportHandler(ih);
						
		initiateStatements();
	}
	
//	/**
//	 * 
//	 * @param conFLAREX Connection to FLAREX
//	 * @param conPDWExpress Connection PDW Express
//	 * @param exp_id  ID of FLAREX Experiment (FLAREX.FS_EXPERIMENTS.EXP_ID)
//	 * @throws SQLException If preparing of Statements or initiating of ImportHandler failed
//	 */
//	public FlarexExperimentImport(Connection conFLAREX, Connection conPDWExpress, String exp_id) throws SQLException {
//		this(conFLAREX,conPDWExpress,exp_id,null);		
//	}



	/**
	 * initiates all statements
	 * @throws SQLException
	 */
	private void initiateStatements() throws SQLException{
		logger.debug("preparing Statements");
		stmtExp = conFLAREX.prepareStatement(QR_EXPERIMENT);
		stmtMat = conFLAREX.prepareStatement(QR_MATERIALS_OF_EXP);
		stmtHyb = conFLAREX.prepareStatement(QR_HYBRIDISATIONS_OF_EXP);
		stmtRst = conFLAREX.prepareStatement(QR_RESULTSETS_OF_EXP);
		stmtSeq = conFLAREX.prepareStatement(QR_BIOSEQUENCES_OF_ADG);
		stmtArr = conFLAREX.prepareStatement(QR_ARRAYS_OF_EXP);
		stmtHrs = conFLAREX.prepareStatement(QR_HYBRESULTS);
		stmtNrs = conFLAREX.prepareStatement(QR_NORMRESULTS_OF_ERS_PAP);
		stmtErs = conFLAREX.prepareStatement(QR_ERS_OF_EXP);
	}
	
	/**
	 * First closes all statements initiated in initiateStatements() <br>
	 * Then closes the used (intern) ImportHandler
	 * 
	 * @throws SQLException If ImportHandler.closeSession() throws SQLException
	 *
	 */
	public final void close() throws SQLException{
		logger.debug("closing Statements");
		DbUtils.closeQuietly(stmtExp);
		DbUtils.closeQuietly(stmtMat);
		DbUtils.closeQuietly(stmtHyb);
		DbUtils.closeQuietly(stmtRst);
		DbUtils.closeQuietly(stmtSeq);
		DbUtils.closeQuietly(stmtArr);
		DbUtils.closeQuietly(stmtHrs);
		DbUtils.closeQuietly(stmtNrs);
		DbUtils.closeQuietly(stmtErs);
		ih.closeSession();
	}
	
	/**
	 * Closes the session
	 * 
	 * @param killOnError If TRUE the seesion will be killed if ImportHandler.closeSession() throws Exception.
	 * @throws SQLException If ImportHandler.closeSession() throws SQLException
	 */
	public final void close(boolean killOnError) throws SQLException{
		try {
			this.close();
		} catch (SQLException e) {
			logger.error("close() "+e);
			if (killOnError){
				logger.error("killing FLAREX import session");
				ih.killSession();
			}
			throw(e);
		}
	}

	/**
	 * @param args
	 * @throws ClassNotFoundException 
	 */
	public static void main(String[] args) throws ClassNotFoundException {
//		BasicConfigurator.configure();
		
        Class.forName("oracle.jdbc.driver.OracleDriver");
        Connection conPDW=null;
        Connection conFLAREX=null;

        try {
			conFLAREX = DriverManager.getConnection("jdbc:oracle:thin:@pgrc-16.ipk-gatersleben.de:1521:genophen","flarex","pgrcarray");
			CallableStatement callStmt =conFLAREX.prepareCall("{ call pdw_secure.pgrc_context.set_username_context('admin') }");
			callStmt.execute();
			callStmt = conFLAREX.prepareCall("{ call pdw_secure.pgrc_context.set_vpd_context }");
			callStmt.execute();			
			conPDW = DriverManager.getConnection("jdbc:oracle:thin:@pgrc-16.ipk-gatersleben.de:1521:pdw","pdw_express_stage","");
			FlarexExperimentImport fi ;
			

			fi = new FlarexExperimentImport(conFLAREX,conPDW,"72",null);			
			fi.importExperiment();
			fi.close(false);
			
			
		} catch (SQLException e) {
			e.printStackTrace();
		} finally{
			DbUtils.commitAndCloseQuietly(conPDW);
			DbUtils.closeQuietly(conFLAREX);
		}
    
		System.out.println("finished.");

	}

	/**
	 * 
	 * @param exp_id
	 * @throws SQLException 
	 */
	public void importExperiment() throws SQLException {
		
		PE_EXPERIMENTS experiment;
		List<PE_ARRAYS> arrays ;
		List<PE_HYBRIDISATIONS> hybridisations;
		List<PE_BIOMATERIALS> materials;
		List<PE_RESULTSETS> resultSets;
		List<PE_BIOSEQUENCES> biosequences;
		try {

			// Experiment
			experiment = readExperiment(exp_id);
			if (experiment ==null){
				logger.warn("Experiment '"+exp_id+"' not found");
				return;
			}				
			String obj_id_exp = experiment.saveObject();
			
			// Array(s) 
			arrays = readArrayOfExp(exp_id);
			logger.info("Number of found Arrays:"+arrays.size());
			PE_ObjectsWrapper<PE_ARRAYS> arrWrapper = new PE_ObjectsWrapperImpl<PE_ARRAYS>(arrays);
			arrWrapper.saveAll();
			conPDWExpress.commit();
								
			// BioSequences
			biosequences=readBioSequences(arrWrapper);
			PE_ObjectsWrapper<PE_BIOSEQUENCES> seqWrapper= new PE_ObjectsWrapperImpl<PE_BIOSEQUENCES>(biosequences);
			seqWrapper.saveAll();
			conPDWExpress.commit();
			
			// Hybridisations
			hybridisations=readHybridisationOfExp(exp_id, obj_id_exp,arrWrapper );
			logger.info("Number of found Hybridisations:"+hybridisations.size());
			PE_ObjectsWrapper<PE_HYBRIDISATIONS> hybWrapper = new PE_ObjectsWrapperImpl<PE_HYBRIDISATIONS>(hybridisations);
			hybWrapper.saveAll();
			conPDWExpress.commit();

			// Materials
			materials=readMaterials(exp_id);
			logger.info("Number of found Materials:"+materials.size());
			PE_ObjectsWrapper<PE_BIOMATERIALS> matWrapper = new PE_ObjectsWrapperImpl<PE_BIOMATERIALS>(materials);
			matWrapper.saveAll();
			conPDWExpress.commit();
			
//			// ResultSets (RawData)
//			resultSets = readRawResultSets(exp_id,hybWrapper,matWrapper);
//			logger.info("Number of found ResultSets:"+resultSets.size());
//			PE_ObjectsWrapper<PE_RESULTSETS> rstWrapper = new PE_ObjectsWrapperImpl<PE_RESULTSETS>(resultSets);
//			rstWrapper.saveAll();
//			conPDWExpress.commit();
//			
//			// Raw data
//			readHybResults(exp_id,rstWrapper,seqWrapper);


			// Normalized data
			if ( pap_id != null)
				readNormalisedData(exp_id, pap_id,seqWrapper,matWrapper,hybWrapper);
		
			conPDWExpress.commit();
			
		} catch (SQLException  e) {
			e.printStackTrace();
			throw e;
		}
		
		
	}

	/**
	 * 
	 * @param exp_id
	 * @param pap_id
	 * @param seqWrapper
	 * @param matWrapper
	 * @param hybWrapper
	 */
	protected void readNormalisedData(String exp_id, 
			String pap_id, 
			PE_ObjectsWrapper<PE_BIOSEQUENCES> seqWrapper,
			PE_ObjectsWrapper<PE_BIOMATERIALS> matWrapper,
			PE_ObjectsWrapper<PE_HYBRIDISATIONS> hybWrapper) {
		logger.info("reading NormalisedData");
		
		ResultSet rsErs=null;
		ResultSet rsNrs = null;
		
		try {
			
			// get all ers of the experiment
			stmtErs.setString(1, exp_id);
			rsErs = stmtErs.executeQuery();
			
			// iterate over all ers of the experiment
			while (rsErs.next()){
				
				String currErs_ID=rsErs.getString("ERS_ID");
				
				// create a resultset for the normalised data
				PE_RESULTSETS currErs = new PE_RESULTSETS();
				currErs.setOBJ_Identifier(IDENTIFIER_PREFIX+"PAP_ID:"+this.pap_id+":ERS_ID:"+currErs_ID, PE_ExternDBEnum.FLAREX);
				currErs.setResultSetType("NormalisedData");
				currErs.setLabelCompound(rsErs.getString("LABELCOMPOUND"));
				String mat_id = matWrapper.getFirstByIdentifier(IDENTIFIER_PREFIX+"LBP_ID:"+rsErs.getString("LBP_ID")).getObj_id();
				String hyb_id = hybWrapper.getFirstByIdentifier(IDENTIFIER_PREFIX+"HYB_ID:"+rsErs.getString("HYB_ID")).getObj_id();
				// TODO NullPointerExp fangen 
				currErs.setFK_RST_HYB_ID(hyb_id); 
				currErs.setFK_RST_MAT_ID(mat_id);
				String rst_id = currErs.saveObject();
				
				//
				ResultDataHandler rstHandler = new ResultDataHandler(this.conPDWExpress,rst_id);
				
				// read normalised data
				stmtNrs.setString(1, currErs_ID);
				stmtNrs.setString(2, pap_id);
				rsNrs = stmtNrs.executeQuery();
				
				while(rsNrs.next()){
					PE_BIOSEQUENCES seq =seqWrapper.getFirstByIdentifier(rsNrs.getString("ACCNUM"));
					rstHandler.addValue(rsNrs.getFloat("DENSITY"), seq);
				}
				rstHandler.close();
				
				if (testImport)
					break;
			}

		} catch (SQLException e) {
			logger.error(e);
			e.printStackTrace();
		}finally{
			DbUtils.closeQuietly(rsErs);
			DbUtils.closeQuietly(rsNrs);
		}		
		
	}


	/**
	 * Maps the flarexDbName to the corresponding ExternDBInterface
	 * 
	 * @param flarexDbName Database name from FLAREX.EXTERNALDATABASES
	 * @return 
	 * @IllegalArgumentException If the flarexDbName was not found.
	 */
	private ExternDBInterface getPE_ExternDB(String flarexDbName) {
		
		PE_ExternDBEnum edb=null;
		if (flarexDbName.equalsIgnoreCase("CR-EST"))
			edb=PE_ExternDBEnum.CREST;
		else{
			String m="Cant't map database:"+flarexDbName+" to PE_ExternDBEnum";
			logger.fatal(m);
			throw new IllegalArgumentException(m);
		}
		
		return edb;
	}




	/**
	 * 
	 * @param rstWrapper
	 * @param seqWrapper 
	 * @throws SQLException 
	 */
	protected void readHybResults(String exp_id,PE_ObjectsWrapper<PE_RESULTSETS> rstWrapper, PE_ObjectsWrapper<PE_BIOSEQUENCES> seqWrapper) throws SQLException {

		logger.info("reading HybResults");
		
		logger.debug("searching for available resultSets");
		
		stmtRst.setString(1, exp_id);
		ResultSet rsRst = stmtRst.executeQuery();
		Set<String> rsetIDs = new HashSet<String>();
		while (rsRst.next())
			rsetIDs.add(rsRst.getString("RSET_ID"));
		DbUtils.closeQuietly(rsRst);
		
		
		Iterator<String> rsetIt = rsetIDs.iterator();
		
		while (rsetIt.hasNext()){
			
			String currRstID = rsetIt.next();
			
			logger.info("Reading FLAREX resultSet rset_id:"+currRstID);
			
			PE_RESULTSETS currRstDen = rstWrapper.getFirstByIdentifier(IDENTIFIER_PREFIX+"RSET_ID:"+currRstID+":Den");
			PE_RESULTSETS currRstBack = rstWrapper.getFirstByIdentifier(IDENTIFIER_PREFIX+"RSET_ID:"+currRstID+":Back");
			
			logger.info("PDWExpress ResultSets Density:"+currRstDen.getOBJ_Identifier()+ " Background:"+currRstBack.getOBJ_Identifier());
			
			ResultDataHandler rsdDen = new ResultDataHandler(this.conPDWExpress, currRstDen.getObj_id());
			ResultDataHandler rsdBack = new ResultDataHandler(this.conPDWExpress, currRstBack.getObj_id());
			
			
			ResultSet rsHrs=null;
			stmtHrs.setString(1, currRstID);
			rsHrs = stmtHrs.executeQuery();
			
			// read Density and Background
			while(rsHrs.next()){
				
				// Map FLAREX 'expernalDatabases' to PE_ExternDBEnum
				ExternDBInterface edb= this.getPE_ExternDB(rsHrs.getString("DATABASENAME"));
				
				PE_BIOSEQUENCES seq = seqWrapper.getFirstByIdentifier(rsHrs.getString("ACCNUM"));
				
				rsdDen.addValue(rsHrs.getFloat("DENSITY"), seq.getObj_id());
				rsdBack.addValue(rsHrs.getFloat("BACKGROUND"), seq.getObj_id());
			}
			
			rsdDen.close();
			rsdBack.close();
			conPDWExpress.commit();

			if (testImport)
				break;
		}
	}//readHybResults




	/**
	 * 
	 * @param arrWrapper
	 * @return
	 * @throws SQLException 
	 */
	protected List<PE_BIOSEQUENCES> readBioSequences(PE_ObjectsWrapper<PE_ARRAYS> arrWrapper) throws SQLException{
		
		logger.info("reading BioSequences from FLAREX");
		
		List<PE_BIOSEQUENCES> allSeq = new Vector<PE_BIOSEQUENCES>(5000,1000);
		
		String currAdg_id=null;
		
		Iterator<PE_ARRAYS> itAdg = arrWrapper.getItems().iterator();
		
		while (itAdg.hasNext()){
			currAdg_id = itAdg.next().getOBJ_Identifier().substring(IDENTIFIER_PREFIX.length()+7);
			logger.info("reading BioSequences of Array ADG_ID:"+currAdg_id);
			ResultSet rs=null;
			try {
				stmtSeq.setString(1, currAdg_id);
				rs=stmtSeq.executeQuery();
				
				while(rs.next()){
					
					// Map FLAREX 'expernalDatabases' to PE_ExternDBEnum
					ExternDBInterface edb= this.getPE_ExternDB(rs.getString("DATABASENAME"));
					
					PE_BIOSEQUENCES seq = new PE_BIOSEQUENCES();
					seq.setOBJ_Identifier(rs.getString("ACCNUM"), edb);
					
					allSeq.add(seq);
					
				}
			} catch (SQLException e) {
				logger.error(e);
				e.printStackTrace();
				throw(e);
			}finally{
				DbUtils.closeQuietly(rs);
			}
		}
		return allSeq;
	}




	/**
	 * 
	 * @param exp_id
	 * @param hybWrapper
	 * @param matWrapper
	 * @return
	 * @throws SQLException 
	 */
	protected List<PE_RESULTSETS> readRawResultSets(String exp_id, PE_ObjectsWrapper<PE_HYBRIDISATIONS> hybWrapper, PE_ObjectsWrapper<PE_BIOMATERIALS> matWrapper) throws SQLException {
		
		logger.info("Reading (raw)ResultSets of Experiment. exp_id:"+exp_id);
		
		List<PE_RESULTSETS> resultSets = new Vector<PE_RESULTSETS>();
		
		ResultSet rs=null;
		
		try {
			stmtRst.setString(1, exp_id);
			rs=stmtRst.executeQuery();
			
			while(rs.next()){

				/*
				 * Density ResultSet
				 */
				{
					PE_RESULTSETS rstDen = new PE_RESULTSETS();
					
					rstDen.setOBJ_Identifier(IDENTIFIER_PREFIX+"RSET_ID:"+rs.getString("RSET_ID")+":Den", PE_ExternDBEnum.FLAREX);
					rstDen.setLabelCompound(rs.getString("LABELCOMPOUND"));
					rstDen.setResultSetType("RawDataDensity");
					rstDen.setFK_RST_HYB_ID(hybWrapper.getFirstByName(rs.getString("HYB_NAME")).getObj_id());				
					rstDen.setFK_RST_MAT_ID(matWrapper.getFirstByName(rs.getString("LBP_NAME")).getObj_id());
					
					resultSets.add(rstDen);
				}

				/*
				 * Background ResultSet
				 */
				{
					PE_RESULTSETS rstBack = new PE_RESULTSETS();
					
					rstBack.setOBJ_Identifier(IDENTIFIER_PREFIX+"RSET_ID:"+rs.getString("RSET_ID")+":Back", PE_ExternDBEnum.FLAREX);
					rstBack.setLabelCompound(rs.getString("LABELCOMPOUND"));
					rstBack.setResultSetType("RawDataBackground");
					rstBack.setFK_RST_HYB_ID(hybWrapper.getFirstByName(rs.getString("HYB_NAME")).getObj_id());				
					rstBack.setFK_RST_MAT_ID(matWrapper.getFirstByName(rs.getString("LBP_NAME")).getObj_id());
					
					resultSets.add(rstBack);
				}
			}
			
		} catch (SQLException e) {
			logger.error(e);
			e.printStackTrace();
			throw(e);
		}finally{
			DbUtils.closeQuietly(rs);
		}
		
		
		return resultSets;
	}




	/**
	 * 
	 * @param exp_id
	 * @return
	 * @throws SQLException 
	 */
	protected List<PE_BIOMATERIALS> readMaterials(String exp_id) throws SQLException {
		
		logger.info("Reading Materials of experiment:"+exp_id);
		
		List<PE_BIOMATERIALS> materials= new Vector<PE_BIOMATERIALS>();
		
		ResultSet rs=null;
		
		try {
			
			stmtMat.setString(1, exp_id);
			rs=stmtMat.executeQuery();
			
			while(rs.next()){
				
			  PE_BIOMATERIALS mat = new PE_BIOMATERIALS();
			  
			  mat.setOBJ_Identifier(IDENTIFIER_PREFIX+"LBP_ID:"+rs.getString("LBP_ID"), PE_ExternDBEnum.FLAREX);
			  mat.setOBJ_Name(rs.getString("NAME"));
			  mat.setOrganism(rs.getString("ORGANISM"),rs.getString("NCBI_SPECIES_ID"),PE_ExternDBEnum.NCBI_Taxonomy);
			  mat.setGenotype(rs.getString("GENOTYPE"));
			  mat.setPhenotype(rs.getString("PHENOTYPE"));
			  mat.setSex(rs.getString("SEX"), rs.getString("SEX"), PE_ExternDBEnum.MGED);
			  mat.setCultivar(rs.getString("CULTIVAR"));
			  mat.setDiseaseStaging(rs.getString("DISEASE_STAGE"));
			  mat.setClinicalTreatment(rs.getString("TREATMENT"));
			  
			  if (!"PO:0009012".equals(rs.getString("PO_DEVSTAGE_ACC")))
				  mat.setDevelopmentalStage(rs.getString("PO_DEVSTAGE_NAME"), rs.getString("PO_DEVSTAGE_ACC"), PE_ExternDBEnum.Plant_Ontology);
			  mat.addDevelopmentalStage(rs.getString("DEVELOPMENT_STAGE"));

			  if (!"PO:0009011".equals(rs.getString("PO_STRUCTURE_ACC")))
				  mat.setOrganismPart(rs.getString("PO_STRUCTURE_NAME"),rs.getString("PO_STRUCTURE_ACC"),PE_ExternDBEnum.Plant_Ontology);
			  
			  
			  if ("y".equals(rs.getString("WILD_TYPE")))
//			  	mat.addGenotype("wild_type");
			  	mat.addGenotype("wild_type","wild_type",PE_ExternDBEnum.MGED);
			  
			  String mat_note=rs.getString("MAT_DESC");
			  String lbp_note=rs.getString("LBP_DESC");			  
			  mat_note = mat_note == null ? "" : mat_note;
			  lbp_note = lbp_note == null ? "" : lbp_note;			  
			  mat.setOBJ_Description(mat_note+"\n\n"+lbp_note);

			  
			  // Age
			  mat.addAge(rs.getString("AGE"));
			  mat.addAge(rs.getString("STAGE_TIME"));
			  
			  //PATHOGEN
			  mat.setPathogenOrganism(rs.getString("PATHOGEN"), rs.getString("NCBI_SPECIES_ID_PATHOGEN"), PE_ExternDBEnum.NCBI_Taxonomy);

			  
			  materials.add(mat);
			}
			
		} catch (SQLException e) {
			logger.error(e);
			e.printStackTrace();
			throw(e);
		}finally{
			DbUtils.closeQuietly(rs);
		}
		
		return materials;
	}



	/**
	 * 
	 * @param exp_id
	 * @return
	 * @throws SQLException
	 */
	protected List<PE_ARRAYS> readArrayOfExp(String exp_id) throws SQLException{
		
		logger.info("Reading Arrays of experiment:"+exp_id);
		
		List<PE_ARRAYS> arrays = new Vector<PE_ARRAYS>();
		
		ResultSet rs=null;
		
		try {
			stmtArr.setString(1, exp_id);
			rs=stmtArr.executeQuery();
			
			while(rs.next()){
				
			  PE_ARRAYS arr = new PE_ARRAYS();
			  
			  arr.setOBJ_Identifier(IDENTIFIER_PREFIX+"ADG_ID:"+rs.getString("ADG_ID"), PE_ExternDBEnum.FLAREX);
			  arr.setOBJ_Name(rs.getString("ADG_NAME"));
			  arr.setOBJ_Description(rs.getString("ADG_DESC"));
			  arr.setArrayTechnology("MacroArray");
			  arr.setArrayManufacturer("IPK");
			  
			  arrays.add(arr);
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw(e);
		}finally{
			DbUtils.closeQuietly(rs);
		}
		
		return arrays;
	}
	

	/**
	 * 
	 * @param exp_id
	 * @param obj_id_exp
	 * @param arrays
	 * @return
	 * @throws SQLException
	 */
	protected List<PE_HYBRIDISATIONS> readHybridisationOfExp(String exp_id, String obj_id_exp, PE_ObjectsWrapper<PE_ARRAYS> arrays) throws SQLException{
		
		logger.info("Reading Hybridisations of FLAREX Experiment:"+exp_id);
		
		List<PE_HYBRIDISATIONS> hybridisation = new Vector<PE_HYBRIDISATIONS>();
		
		ResultSet rs=null;
		
		try {
			stmtHyb.setString(1, exp_id);
			rs=stmtHyb.executeQuery();
			
			while (rs.next()){
				PE_HYBRIDISATIONS hyb = new PE_HYBRIDISATIONS();
				hyb.setOBJ_Identifier(IDENTIFIER_PREFIX+"HYB_ID:"+rs.getString("HYB_ID"), PE_ExternDBEnum.FLAREX);
				hyb.setOBJ_Name(rs.getString("HYB_NAME"));
				hyb.setOBJ_Description(rs.getString("HYB_DESC"));
				hyb.setReplicateIndex(rs.getString("REP_INDEX"));
				hyb.setHybCreationDate(rs.getString("HYB_CREATIONDATE"));
				hyb.setHybSlide(rs.getString("MBR_NAME"));
				
				// get Array ID within PDWExpress				
				PE_Object o= arrays.getFirstByName(rs.getString("ADG_NAME"));
				if (o == null){
					String m="Array '"+rs.getString("ADG_NAME")+"' not found.";
					logger.fatal(m);
					throw new NullPointerException(m);
				}
				String obj_id_arr = o.getObj_id();
				
				
				hyb.setFK_HYB_EXP_ID(obj_id_exp);
				hyb.setFK_HYB_ARY_ID(obj_id_arr);
				
				hybridisation.add(hyb);
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw(e);
		}finally{
			DbUtils.closeQuietly(rs);
		}
		
		
		return hybridisation;
	}
	
	/**
	 * 
	 * @param exp_id FLAREX Experiment ID 
	 * @return experiment The read experiment. NULL if no data found. 
	 * @throws SQLException 
	 */
	protected PE_EXPERIMENTS readExperiment(String exp_id) throws SQLException{
		
		logger.info("Reading Experiment exp_id:"+exp_id);
		
		PE_EXPERIMENTS experiment =null;
		
		ResultSet rs =null;
		
		try {
			stmtExp.setString(1, exp_id);
			rs = stmtExp.executeQuery();
			
			if(rs.next()){
				
				experiment = new PE_EXPERIMENTS();
				
				experiment.setOBJ_Identifier(IDENTIFIER_PREFIX+"EXP_ID:"+rs.getString("EXP_ID"), PE_ExternDBEnum.FLAREX);
				experiment.setOBJ_Name(rs.getString("NAME"));
				
				experiment.setDataSource("FLAREX");
				experiment.setLaboratory("IPK");
				experiment.setOBJ_Description(rs.getString("DESCRIPTION"));
				experiment.setExperimenter(rs.getString("PUBLISHER"));
			}
			
		} catch (SQLException e) {
			logger.error("Error while reading experiments data.");
			e.printStackTrace();
			throw(e);
		}finally{
			DbUtils.closeQuietly(rs);
		}
		return experiment;
		
	}
	
	
	private void test(){
		String a[]=new String[5];
		
		
		for (String string : a) {
			
		}
	}
	
}// class
