package de.pdw_express.parser.sged;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import de.ipk.pdwexpress.peobjects.PE_ARRAYS;
import de.ipk.pdwexpress.peobjects.PE_BIOMATERIALS;
import de.ipk.pdwexpress.peobjects.PE_BIOSEQUENCES;
import de.ipk.pdwexpress.peobjects.PE_EXPERIMENTS;
import de.ipk.pdwexpress.peobjects.PE_ExternDBEnum;
import de.ipk.pdwexpress.upload.AbstractPEObject;
import de.ipk.pdwexpress.upload.ImportHandler;
import de.pdwexpress.parser.common.MatrixParser;

public class Import {

	ImportHandler importHandler;
	Connection con;
	
	String dataFileName;
	String resultsetFileName;
	String materialFileName;
	String hybridisationFileName;
	String experimentFileName;
	String arrayForeignKey;
	
	
	String experiment_PK;
	Hashtable <String, String> matIDs;
	
	public Import(ImportHandler importHandler, Connection con)throws SQLException {
		super();
		// TODO Auto-generated constructor stub
		
		
		this.importHandler = importHandler;
		this.con = con;
		
		PE_ARRAYS myArray = new PE_ARRAYS();
		myArray.setArrayManufacturer("TIGR");
		myArray.setArrayTechnology("cDNA Microarray");
		//myArray.setOBJ_Identifier("A-TIGR-19",PE_ExternDBEnum.ArrayExpress);
		myArray.setOBJ_Name("TIGR potato 10k cDNA array v2");
		//arrayForeignKey=myArray.saveObject();
		
	}



/*
 * Setter methods for Data Files
 */
	public void setDataFileName(String dataFileName) {
		this.dataFileName = dataFileName;
	}




	public void setExperimentFileName(String experimentFileName) {
		this.experimentFileName = experimentFileName;
	}




	public void setMaterialFileName(String materialFileName) {
		this.materialFileName = materialFileName;
	}





	


/*
 *  Import the Experiment data
 */
	
	public void insertExperiment(String experiment_ID) throws IOException,NoSuchMethodException, ClassNotFoundException, SQLException, FileNotFoundException{

		
		String expObjKey=new String();
		try{
		
			
			MatrixParser<PE_EXPERIMENTS> ap = new MatrixParser<PE_EXPERIMENTS>(new PE_EXPERIMENTS());
			List<PE_EXPERIMENTS> myExp = ap.parseFile(new File(experimentFileName));
		
			
		
			for (PE_EXPERIMENTS ex:myExp ){
				ex.setOBJ_Identifier(experiment_ID, PE_ExternDBEnum.SGED);
				ex.saveObject();
				expObjKey=ex.getObj_id();
			}
			
		}catch (SQLException e){
			con.rollback();
			throw e;
		}
		experiment_PK=expObjKey;
		
	}


	


public Hashtable<String, String> insertMaterials()
	throws ClassNotFoundException, SQLException, FileNotFoundException, IOException, NoSuchMethodException{

		matIDs= new Hashtable<String, String>();
	
		//Class.forName("oracle.jdbc.driver.OracleDriver");
		//Connection con = DriverManager.getConnection("jdbc:oracle:thin:@pgrc-16.ipk-gatersleben.de:1521:pdw","pdw_express_stage","");
	try{
		//ImportHandler importHandler = new ImportHandler(con);
		AbstractPEObject.setImportHandler(importHandler);
	
		MatrixParser<PE_BIOMATERIALS> ap = new MatrixParser<PE_BIOMATERIALS>(new PE_BIOMATERIALS());
		List<PE_BIOMATERIALS> myExp = ap.parseFile(new File(materialFileName));
	
		for (PE_BIOMATERIALS ex:myExp ){
			
			ex.saveObject();
			
			matIDs.put(ex.getOBJ_Name(),ex.getObj_id());
		}
		
	}catch (SQLException e){
		con.rollback();
		throw e;
	}
	
	
	return matIDs;
}











	public void insertData(Hashtable matIDs)throws FileNotFoundException, IOException, SQLException{
		BufferedReader in = new BufferedReader(new FileReader(dataFileName));
		String inputline;
		String[] splitline;
		Vector<TIGR_Experiment> tigrExperiments = new Vector<TIGR_Experiment>();
		Hashtable hybIDs= new Hashtable();
		Vector<String[]> resSetIDs= new Vector<String[]>();
		
		
		int numberOfColumns;  //number of Columns of the tsv-file
		int numberOfEntries;  //number of entries for one TIGRExperiment
		int numberOfTIGRExperiments;     //number of TIGR_Experiments in the file. An Experiment in the name convention of TIGR summons all Hybridisations of one unit of technical replicates
		
		
		
		
		inputline = in.readLine();
		splitline = inputline.split("\t", -2);
		for (int i=1; i<splitline.length; i++){
			if (splitline[i].equalsIgnoreCase("")!=true ){
				
				TIGR_Experiment tE = new TIGR_Experiment();
				String hybName = splitline[i];
				tE.name=splitline[i];
				tE.firstColumn=i;
				
				/*
				 * tissue
				 */
				if (hybName.matches("[a-zA-Z0-9]*leaves[a-zA-Z0-9]*")){
					tE.tissue="leaves";
				}else if (hybName.matches("[a-zA-Z0-9]*stolons[a-zA-Z0-9]*")){
					tE.tissue ="stolons";
				}
				
				/*
				 * days of treatment
				 */
				if (hybName.startsWith("2d")){
					tE.setDay("2");
				}else if (hybName.startsWith("4d")){
					tE.setDay("4");
				}else if (hybName.startsWith("8d")){
					tE.setDay("8");
				}else if (hybName.startsWith("12d")){
					tE.setDay("12");
				}else if (hybName.startsWith("16d")){
					tE.setDay("16");
				}
				
				/*
				 * biological Replicate
				 */
				if(hybName.endsWith("A")){
					tE.setBioReplicateNumber("1");
				}else if (hybName.endsWith("B")){
					tE.setBioReplicateNumber("2");
				}else if (hybName.endsWith("FD")){
					tE.setBioReplicateNumber("3");
				}
				/*
				 * Label Cy3/Cy5 or Flip Dye
				 */
				if(hybName.endsWith(("FD"))){
					tE.setTestLabel("Cy5");
					tE.setControlLabel("Cy3");
				}else{
					tE.setTestLabel("Cy3");
					tE.setControlLabel("Cy5");					
				}
				
				
				tE.insertIntoDB(experiment_PK,matIDs );
				
				String[] g = new String[2];
				g[1]=tE.getRes1_FK();
				g[0]=new Integer(tE.firstColumn).toString();
				resSetIDs.add(g);
				
				g = new String[2];
				g[1]=tE.getRes2_FK();
				g[0]=new Integer(tE.firstColumn+1).toString();
				resSetIDs.add(g);
				
				g = new String[2];
				g[1]=tE.getRes3_FK();
				g[0]=new Integer(tE.firstColumn+2).toString();
				resSetIDs.add(g);
				
				g = new String[2];
				g[1]=tE.getRes4_FK();
				g[0]=new Integer(tE.firstColumn+3).toString();
				resSetIDs.add(g);
				
				

				
				
			//	tigrExperiments.add(tE);
			}

		}	
		
		in.readLine();
		inputline=in.readLine();
		
		PreparedStatement pst = con.prepareStatement("insert into results_data_t values "+
		"?,?,?");
		
		while (inputline != null){
			
			splitline = inputline.split("\t");
			
			PE_BIOSEQUENCES seq = new PE_BIOSEQUENCES();
			seq.setOBJ_Identifier(splitline[0], PE_ExternDBEnum.SGED);
			String seqFK=seq.saveObject();
			
			for(Enumeration myenum=resSetIDs.elements();myenum.hasMoreElements();){
			String[] h=(String[])myenum.nextElement();	
				int theindex= Integer.parseInt(h[0]);
				int rset_fk = Integer.parseInt(h[1]);

				pst.setInt(1,rset_fk);
				pst.setInt(2, Integer.parseInt(seqFK));
				pst.setInt(3, Integer.parseInt(splitline[theindex]));
				
				//System.out.println( splitline[0] +"   "+theindex+"  "+splitline[theindex]);
			
			
			}
			
			
			inputline= in.readLine();
		}
		pst.close();
		
			
		
		
		
		
		
		
	
	in.close();
	
	}

	
	

}