package de.pdwexpress.parser.afgc_atgenexpress;


import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.FileAppender;
import org.apache.log4j.PatternLayout;

import java.io.File;
import java.io.IOException;
import java.io.FileReader;
import java.io.BufferedReader;
import java.util.*;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import de.ipk.pdwexpress.peobjects.PE_EXPERIMENTS;
import de.ipk.pdwexpress.peobjects.PE_HYBRIDISATIONS;
import de.ipk.pdwexpress.peobjects.PE_BIOMATERIALS;
import de.ipk.pdwexpress.peobjects.PE_ARRAYS;
import de.ipk.pdwexpress.upload.ImportHandler;
import de.ipk.pdwexpress.upload.AbstractPEObject;
import de.ipk.pdwexpress.tableObjects.EditingSession;

/**
 * @author Andreas Stephanik
 */
public class AFGC_AtGenExpress_ImportController {


    PE_EXPERIMENTS experiment;

    boolean experimentSaved;

    PE_HYBRIDISATIONS hybridisation;

    ImportHandler importHandler;

    Connection connection;

    AFGC_AtGenExpress_ReadmeParser readmeParser;

    AFGC_AtGenExpress_ResultSetParser originalDataResultSetParser;

    boolean logging = true;

    /**
     * Set to true if a file 'expr_set_dir_file' shall be
     * used to define the expression-set directories instead
     * scanning all subdirs.
     * @see de.pdwexpress.parser.afgc_atgenexpress.DirFileGenerator#expr_set_dir_file
     */
    boolean useExpressionSetDirsFile = false;



    public AFGC_AtGenExpress_ImportController() throws ClassNotFoundException, SQLException {
        connection = createConnection();
    }


    public AFGC_AtGenExpress_ImportController(ImportHandler importHandler, Connection connection) {
        this.importHandler = importHandler;
        this.connection = connection;
    }


    public static Connection createConnection() throws SQLException, ClassNotFoundException {
        Class.forName("oracle.jdbc.driver.OracleDriver");
        return DriverManager.getConnection("jdbc:oracle:thin:@pgrc.ipk-gatersleben.de:1521:pdw", "pdw_express_stage", "");
    }

    /**
     * Checks if the supplied parent dir has a File.separator at the end.
     * Calls getFileNames() in order to determine the _README.txt files.
     * Calls importExpressionSet() for each expression set (experiment) for importing the data into pdw_express_stage.
     *
     * @param parentDir The parent dir of ExpressionSet_XXXXX directories or the file where ExpressionSet_XXXX dirs are defined
     */
    public void importAllExpressionSets(String parentDir) {
        LinkedHashMap experimentsAndHybs;
        if(useExpressionSetDirsFile)
            experimentsAndHybs = getExpressionSetDirectoriesAndFileNamesByDirFile(parentDir);
        else {
            if( !File.separator.equals(parentDir.substring(parentDir.length()-1)) )
                parentDir += File.separator;
            experimentsAndHybs = getExpressionSetDirectoriesAndFileNames(parentDir); // returns the ExpressioNSet_XXXXX dirs and contained _README.txt file names
        }
        if(experimentsAndHybs == null || experimentsAndHybs.size() == 0) {
            System.out.println("No files found in " + parentDir);
            return;
        }
        String experiment;
        String[] files;
        for (Iterator<String> it = experimentsAndHybs.keySet().iterator(); it.hasNext(); ) {
            experiment = it.next();
            files = (String[]) experimentsAndHybs.get(experiment);
            System.out.println("\n*****************************");
            System.out.println("Expression set directory:\t" + experiment);
            if(files != null) {
                importExpressionSet(experiment, files);
            }
            else
                System.out.println("No files for experiment " + " found in " + parentDir);
            //afgcEntryWriter.closeConnection();
        }
    }

    public void importExpressionSet(String expressionSetDir) {
        if( !File.separator.equals(expressionSetDir.substring(expressionSetDir.length()-1)) )
            expressionSetDir += File.separator;
        importExpressionSet(expressionSetDir, getFileNames(expressionSetDir));
    }

    /**
     *
     * @param expressionSetDir  The expressions et directory where the _README.txt files, .cel files, original AFGC data files, and normalized data files are located
     * @param files     Absolute file names of the _README.txt files
     */
    public void importExpressionSet(String expressionSetDir, String[] files) {
        if(files != null) {
            String exprAcc = expressionSetDir.substring(expressionSetDir.lastIndexOf("_")+1, expressionSetDir.length()-1); // use the accession number from directory name ExpressionSet_XXXX
            if(logging) {
                FileAppender fAppender = null; // log4j
                BasicConfigurator.resetConfiguration(); // log4j
                try {
                    fAppender = new FileAppender( new PatternLayout(PatternLayout.TTCC_CONVERSION_PATTERN), "import_"+ exprAcc + ".log", false );
                    BasicConfigurator.configure(fAppender); // log4j
                }
                catch (IOException e) {
                    e.printStackTrace();
                    BasicConfigurator.configure(); // log4j
                }
            }
            System.out.println("Starting import of experiment " + expressionSetDir + " with " + files.length + " hybridisations");
            try {
                EditingSession editingSession = new EditingSession("Import of ExpressionSet "+exprAcc, "TAIR");
                importHandler = new ImportHandler(connection, editingSession); // start a session here (see ImportHandler) for each experiment
            }
            catch (SQLException e) { // can not create ImportHandler with EditingSession
                System.err.println("importExpressionSet() A");
                e.printStackTrace();
                try {
                    importHandler.killSession();
                }
                catch (SQLException e1) {
                    System.err.println("importExpressionSet() B");
                    e1.printStackTrace();
                }
                return;
            }
            AbstractPEObject.setImportHandler(importHandler);

            experiment = new PE_EXPERIMENTS();
            //experiment.setDataSource("TAIR");
            experimentSaved = false; // in order to save the experiment with all its hybs only once

            for (int i = 0; i < files.length; i++) {  // iterate thru all hyb files, which also contain data about the experiment (so experiment data is redundant in each hyb file)
                System.out.println("Annotation File:\t" + files[i]);
                try {
                    importHybAnnotationData(expressionSetDir, files[i]); // import the annotation data of the *_README.txt files
                    importHybResults();  // import the hyb results
                }
                catch (Exception e) { // something went wrong while importing or saving the data
                    try {
                        System.err.println("Killing import session ...");
                        importHandler.killSession(); // kill the import session
                    }
                    catch (SQLException e1) {
                        System.err.println("importExpressionSet() C");
                        e1.printStackTrace();
                    }
                    System.err.println("importExpressionSet() D");
                    e.printStackTrace();
                    try {
                        connection.commit(); // commit changes made by killing the session. Otherwise, the import of the subsequent experiment is not visible until the commit of the first hybridisation.
                    }
                    catch (SQLException e1) {
                        e1.printStackTrace();
                    }
                    return; // do not import further hybs of this expression set (experiment)
                }
                try {
                    connection.commit(); // commit the imported data of the current hybridisation
                }
                catch (SQLException e) {
                    System.err.println("importExpressionSet() F");
                    e.printStackTrace();
                }
            } // for each Hybridisation (_README.txt, original, tair-normalized) file
            try {
                importHandler.closeSession(); // close the successful import session of the whole expression set
                connection.commit(); // commit the imported data
                System.out.println("Experiment " + exprAcc + " imported.");
            }
            catch (SQLException e) {
                System.err.println("importExpressionSet() G");
                e.printStackTrace();
            }
        }
        else {
            System.out.println("No files supplied. Expression set dir is " + expressionSetDir);
            return;
        }
    }


    /**
     * Imports the annotation data from the the supplied _README.txt file.
     * @param expressionSetDir
     * @param file      Absolute file name the the _README.txt file.
     * @throws Exception
     */
    void importHybAnnotationData(String expressionSetDir, String file) throws Exception {
        readmeParser = new AFGC_AtGenExpress_ReadmeParser(expressionSetDir, file, experiment);;
        try {
            hybridisation = readmeParser.parseAndImportHybData();
            saveHybAnnotationData(); // save the parsed hybridisation data into the DB. It works as long as the result sets are separately stored in files for each hyb. Otherwise all hybs have to be collected.
        }
        catch (Exception e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            readmeParser.closeParser();  // close the BufferedReader
            throw e;  // and throw an exception for killSession()
        }
        readmeParser.closeParser(); // close the BufferedReader
    }

    void saveHybAnnotationData() throws SQLException {
        // experiment-hybs, array-hyb, materials-hyb, hyb-resultsets
        String experimentId;
        if(experimentSaved)
            experimentId = experiment.getObj_id();
        else {
            experimentId = experiment.saveObject();   // experiment is created in this class
            experimentSaved = true;   // an experiment is only saved once for all its hybs
        }
        hybridisation.setFK_HYB_EXP_ID(experimentId);  // the parsed hybridisation (AFGC_AtGenExpress_ReadmeParser) corresponds to an experiment

        PE_ARRAYS array = readmeParser.getArray();
        String arrayId = array.saveObject();
        hybridisation.setFK_HYB_ARY_ID(arrayId);

        String hybridisationId = hybridisation.saveObject(); // hybridisation is created in AFGC_AtGenExpress_ReadmeParser and assigned in this class in method importHybData()

        PE_BIOMATERIALS biomaterial_1 = readmeParser.getBiomaterial_1();
        if(biomaterial_1 != null) {
            String biomatId = biomaterial_1.saveObject();
        }
        if(readmeParser.isAFGCFlag()) {
            PE_BIOMATERIALS biomaterial_2 = readmeParser.getBiomaterial_2();
            if(biomaterial_2 != null) {
                String biomatId = biomaterial_2.saveObject();
            }
        }
    }

    void importHybResults() throws Exception {
        originalDataResultSetParser = null;
        if(readmeParser.isAFGCFlag()) {
            if(readmeParser.getOriginalDataFileName() != null && readmeParser.getOriginalDataFileName().trim().length() > 0) {
                originalDataResultSetParser = new AFGC_AtGenExpress_ResultSetParser(readmeParser.getOriginalDataFileName(),
                        readmeParser, connection);
                originalDataResultSetParser.parseFileAndSaveData(); // we need some information in while parsing the normalised data, e.g. the FKs of the result sets of the original data
            }
        }
        // import normalized data
        if(readmeParser.getTairNormalizedDataFileName() != null && readmeParser.getTairNormalizedDataFileName().trim().length() > 0) {
            new AFGC_AtGenExpress_ResultSetParser(readmeParser.getTairNormalizedDataFileName(),
                    readmeParser, originalDataResultSetParser, connection).parseFileAndSaveData();
        }
    }


    /**
     * Determines the file names of *_README.txt files within a parent directory that
     * contains sub directories with pattern ExpressionSet_* (the asterix stands for the
     * experiment name, either a number or ME+number). The *_README.txt files contain
     * the experiment and hybridisation annotaions.<br/>
     * <br/>
     * Each ExpressionSet has
     * a triplet *_original.txt, *_README.txt, *_tair.txt   for Microarrays of AFGC or
     * a triplet *_.cel, *_README.txt, *.txt                for Affymetrix-arrays of AtGenExpress
     *
     * @param parentDirectory     Parent directory for ExpressionSet_* sub directories
     * @return HashMap with full path of the ExpressionSet_XXXX directory names and String array with full file names of the *_README.txt files within the sub directory ExpressionSet_*.
     */
    public static LinkedHashMap<String,String[]> getExpressionSetDirectoriesAndFileNames(String parentDirectory) {
        try {
            System.out.println("Parent dir for expression sets:" + parentDirectory);
            File[] expressionSetDirs = new File(parentDirectory).listFiles(); // all files/directories within the parent directory
            LinkedHashMap<String,String[]> expressionSetDirsAndFileNames = new LinkedHashMap<String, String[]>();
            for (int i = 0; i < expressionSetDirs.length; i++) {
                File exprSetDir = expressionSetDirs[i];
                if(!exprSetDir.isFile()) { // expression set sub directory
                    //System.out.println("Found directory '" + file.getAbsolutePath() + "'");
                    int index = exprSetDir.getName().lastIndexOf("ExpressionSet_");   // behind the "_" only numbers or ME+numbers are allowed
                    if( index > -1 ) {
                        String[] fileNames = getFileNames(exprSetDir.getAbsolutePath() + File.separator); // look for the _README.txt files in the expression set dir
                        if(fileNames != null && fileNames.length > 0) // add files to expression set dir only if at least one was found
                            expressionSetDirsAndFileNames.put(exprSetDir.getAbsolutePath() + File.separator, fileNames);
                    }
                }
            }
            return expressionSetDirsAndFileNames;
        }
        catch (Exception e) {
            System.err.println("AFGC_AtGenExpress_ImportController.getFileNames():" + e);
            e.printStackTrace();
        }
        return null;
    }

    public static  LinkedHashMap<String,String[]> getExpressionSetDirectoriesAndFileNamesByDirFile(String exprSetDirFile) {
        try {
            BufferedReader br = new BufferedReader(new FileReader(exprSetDirFile));
            String exprSetDir = br.readLine();
            LinkedHashMap<String,String[]> expressionSetDirsAndFileNames = new LinkedHashMap<String, String[]>();
            String[] fileNames;
            while(exprSetDir != null) {
                exprSetDir = exprSetDir.trim();
                if( !exprSetDir.startsWith("#") && exprSetDir.length() > 0 ) { // skip uncommented (#) and empty lines
                    fileNames = getFileNames(exprSetDir); // look for the _README.txt files in the expression set dir
                    if(fileNames != null && fileNames.length > 0) // add files to expression set dir only if at least one was found
                        expressionSetDirsAndFileNames.put(exprSetDir, fileNames);
                }
                exprSetDir = br.readLine();
            }
            br.close();
            return expressionSetDirsAndFileNames;
        }
        catch (Exception e) {
            System.err.println("AFGC_AtGenExpress_ImportController.getExpressionSetDirectoriesAndFileNamesByDirFile():" + e);
        }
        return null;
    }


    /**
     * Scans expression set directory for *_README.txt files, which contain
     * annotation data.
     *
     * @param expressionSetDir The directory of an expression set (ExpressionSet_*).
     * @return list of  absolute file names of _README.txt files.
     */
    public static String[] getFileNames(String expressionSetDir) {
        try {
            System.out.println("Scanning directory " + expressionSetDir + " ...");
            File[] files = new File(expressionSetDir).listFiles(); // all files within the directory
            TreeSet<String> treeSet = new TreeSet<String>();
            for (int i = 0; i < files.length; i++) {
                File file = files[i];
                if(file.isFile()) {
                    String fileName = file.getName();
                    if( fileName.length() > 11 && "_README.txt".equals(fileName.substring(fileName.length()-11)) ) // only the .txt files are desired
                        treeSet.add(file.getAbsolutePath());
                }
            }
            System.out.println("Number of found *_README.txt files in " + expressionSetDir + ": " + treeSet.size());
            if(treeSet.size() > 0) {
                String[] filenames = new String[treeSet.size()];
                Iterator<String> it = treeSet.iterator();
                int i = 0;
                while (it.hasNext()) {
                    filenames[i++] = it.next();
                }
                return filenames;
            }
        }
        catch (Exception e) {
            System.err.println("AFGC_AtGenExpress_ImportController.getFileNames():" + e);
        }
        return null;
    }

    public void closeConnection() {
        try {
            connection.close();
        }
        catch (SQLException e) {
            e.printStackTrace();
        }
    }


    public void setLogging(boolean logging) {
        this.logging = logging;
    }

    public void setUseExpressionSetDirsFile(boolean useExpressionSetDirsFile) {
        this.useExpressionSetDirsFile = useExpressionSetDirsFile;
    }

    static void printUsage() {
        System.out.println("Usage: java AFGC_AtGenExpress_ImportController [-nologging] 'expression_set_dirs.txt'|<dirName>\n");
        System.out.println("\t The <dirName> defines the parent  directory where the expression sets of AFGC (ExpressionSet_XXX) are located.");
        System.out.println("\t The expression sets of AFGC (ExpressionSet_XXX) contains the data files (triplets *_original.txt, *_README.txt, *_tair.txt).");
        System.out.println("\t For example: The directory /data/Datasets/  contains the sub-directories with the expression sets (ExpressionSet_1, ExpressionSet_10, ...)\n");
        System.out.println("\t\t All expression set sub-directories are scanned for the data files.");
        System.out.println("\t\t Hence, the dir has to be /data/Datasets/\n");
        System.out.println("\t Alternatively, you can specify a file 'expression_set_dirs.txt' which contains all dirs of expression sets to be imported line-wise.");
        System.out.println("\t\t You can use DirFileGenerator to generate this file 'expression_set_dirs.txt' and modify (delete dirs) according to your import expression-set list.\n\n");
        System.out.println("\t The option -nologging disables the logging (log4j) of database-related actions into logging files, which have a size of several mega bytes for each experiment!");
        System.out.println("\t\t Beside log4j-logging, ordinary System.out or System.err are used (but rarely).");
        System.out.println("");
    }




    public static void main(String[] args) {
        if(args.length == 0) {
            AFGC_AtGenExpress_ImportController.printUsage();
            System.exit(0);
        }
        /*
        try {
            FileAppender fAppender = new FileAppender( new PatternLayout(PatternLayout.TTCC_CONVERSION_PATTERN), "import.log", false ); // overwrite existing log file
            //BasicConfigurator.configure(fAppender); // log4j
        }
        catch (IOException e) {
            e.printStackTrace();
            BasicConfigurator.configure(); // log4j
        }
        */

        AFGC_AtGenExpress_ImportController im = null;
        int di = 0; // index within args for directory
        try {
            try {
                im = new AFGC_AtGenExpress_ImportController(); // try to open a JDBC connection
                if(args[0].equalsIgnoreCase("-nologging")) {
                    if(args.length == 1) {
                        AFGC_AtGenExpress_ImportController.printUsage();
                        im.closeConnection();
                        System.exit(0);
                    }
                    im.setLogging(false);
                    di = 1; // the directory is defined as second arg
                }
                else
                    im.setLogging(true);

                if( args[di].endsWith(DirFileGenerator.expr_set_dir_file) ) {
                    im.setUseExpressionSetDirsFile(true);  // use the file 'expression_set_dirs.txt' for the definition of expression sets to be imported
                    im.importAllExpressionSets(args[di]);
                }
                else if(args[di].contains("ExpressionSet_"))
                    im.importExpressionSet(args[di]);  // just import one ExpressionSet_* dir, e.g. D:\Downloads\AtGenExpress\Datasets\ExpressionSet_ME00342
                else
                    im.importAllExpressionSets(args[di]); // Import all ExpressionSet_* sub dirs
            }
            catch (SQLException e) {
                System.out.println("Connection could not be established");
            }
            catch (ClassNotFoundException e) {
                System.out.println("ClassNotFoundException: " + e);
            }
        }
        catch (Exception e) {
            System.err.println("AFGC_AtGenExpress_ImportController.main():" + e);
        }
        im.closeConnection();
    }
}
