package de.pdwexpress.parser.common;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.lang.reflect.Array;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import de.ipk.pdwexpress.peobjects.PE_ARRAYS;
import de.ipk.pdwexpress.peobjects.PE_BIOMATERIALS;
import de.ipk.pdwexpress.peobjects.PE_EXPERIMENTS;
import de.ipk.pdwexpress.peobjects.PE_HYBRIDISATIONS;
import de.ipk.pdwexpress.peobjects.PE_RESULTSETS;
import de.ipk.pdwexpress.upload.ExternDBInterface;
import de.ipk.pdwexpress.upload.ImportHandler;

public class AnnotationFileParser {
	
	Connection con;
	ImportHandler importHandler;
	String path;
	String experimentAnnotationFileName;
	
	ExternDBInterface array_edb;
	ExternDBInterface edb;
	
	// The Primary Keys
	String experimentID;
	String arrayID;
	Hashtable <String, String> materialIDs;
	Hashtable <String, String> hybridisationIDs;
	Hashtable <String, String> resultsetIDs;
	
	// Number of Columns each topic
	String numberOfHybridisations;
	String numberOfResultsets;	
	String numberOfBiomaterials;
	
	//Names of Hybridisation and Resultsets
	//Vector<String> hybNames;
	Vector<String> rsetNames;
	
	Hashtable <String, String> res2mat;
	Hashtable <String, String> res2hyb;
	
	
	
	
	
	
public AnnotationFileParser(Connection con, String path, String fileName, ImportHandler importHandler,ExternDBInterface edb, ExternDBInterface array_edb){
		System.out.println("AnnotationFileParser is initalized.");
		System.out.println("parsing file: "+path+fileName);
		this.con= con;
		this.importHandler = importHandler;
		
		this.path=path;
		this.edb=edb;
		this.array_edb=array_edb;
		experimentAnnotationFileName=fileName;
		
		
		res2mat= new Hashtable<String, String>();
		res2hyb= new Hashtable<String, String>();
		hybridisationIDs= new Hashtable<String, String>();
		materialIDs=new Hashtable<String, String>();
		resultsetIDs=new Hashtable<String, String>();
		//hybNames = new Vector<String>();
		rsetNames= new Vector<String>();
				
	}
	
	
	
	
	
	public void insertAnnotations(){
		System.out.println("starting method: AnnotationFileParser.insertAnnotations()");
		try{
			BufferedReader in = new BufferedReader(new FileReader(path+experimentAnnotationFileName));
			
			
			 experimentID=readExperimentAnnotation(in).saveObject();
			 in.close();
			 in = new BufferedReader(new FileReader(path+experimentAnnotationFileName));	
			 arrayID= readArrayAnnotation(in).saveObject();
			 in.close();
			 in = new BufferedReader(new FileReader(path+experimentAnnotationFileName));
			 
			
			for (Enumeration<PE_HYBRIDISATIONS> myenum=readHybridisationAnnotation(in).elements(); myenum.hasMoreElements();){
				PE_HYBRIDISATIONS peh=myenum.nextElement();
				peh.setFK_HYB_ARY_ID(arrayID);
				peh.setFK_HYB_EXP_ID(experimentID);
				String hybID=peh.saveObject();
				//System.out.println(peh.getOBJ_Identifier()+"   "+hybID);
				hybridisationIDs.put(peh.getOBJ_Name(),hybID);
				
			}
			 in.close();
			 in = new BufferedReader(new FileReader(path+experimentAnnotationFileName));
			for (Enumeration<PE_BIOMATERIALS> myenum=readBioMaterialAnnotation(in).elements(); myenum.hasMoreElements();){
				PE_BIOMATERIALS peb =myenum.nextElement();
				materialIDs.put(peb.getOBJ_Identifier(), peb.saveObject());
			}
			 in.close();
			 in = new BufferedReader(new FileReader(path+experimentAnnotationFileName));
			
			for (Enumeration<PE_RESULTSETS> myenum=readResultsetAnnotation(in).elements(); myenum.hasMoreElements();){
				PE_RESULTSETS per =myenum.nextElement();
				//System.out.println(hybridisationIDs.get(res2hyb.get(per.getOBJ_Identifier())));
				per.setFK_RST_HYB_ID(hybridisationIDs.get(res2hyb.get(per.getOBJ_Identifier())));
				per.setFK_RST_MAT_ID(materialIDs.get(res2mat.get(per.getOBJ_Identifier())));
				//System.out.println((per.getOBJ_Name()));
				resultsetIDs.put(per.getOBJ_Identifier(), per.saveObject());
			}
			in.close();
		}catch(IOException e){System.out.println("FroschNotFoundException: "+e);}
		 catch(SQLException e){System.out.println("AnnotationFileParser "+e);}
		
	}
	
	
		private PE_EXPERIMENTS readExperimentAnnotation(BufferedReader in)throws IOException{
			System.out.println("parsing Experiment Annotation");
			PE_EXPERIMENTS pee = new PE_EXPERIMENTS();
			
			String inputline = in.readLine();
			String[] splitline;
			while (inputline != null){
				if (inputline.startsWith("<Experiment")){
				
					
					
					inputline = in.readLine();
					while (inputline.startsWith("<")!=true){				
						splitline = inputline.split("\t");
						
						if (splitline[0].equalsIgnoreCase("OBJ_Identifier")&& splitline[1].equalsIgnoreCase("")!=true){
							pee.setOBJ_Identifier(splitline[1], edb);
							System.out.println(splitline[1]);
						}						
							
						
						
						if (splitline[0].equalsIgnoreCase("OBJ_Name")&& splitline[1].equalsIgnoreCase("")!=true){
							pee.setOBJ_Name(splitline[1]);
							System.out.println(splitline[1]);
						}						
						
						try{
							if (splitline[0].equalsIgnoreCase("OBJ_Description")){
								pee.setOBJ_Description(splitline[1]);
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("DataSource")){
								pee.setDataSource(splitline[1]);
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						try{
							if (splitline[0].equalsIgnoreCase("ExperimentDesignType")){
								pee.setExperimentDesignType(splitline[1]);
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("Experimenter")){
								pee.setExperimenter(splitline[1]);
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("Laboratory")){
								pee.setLaboratory(splitline[1]);
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						

						
						
						
						if (splitline[0].equalsIgnoreCase("NumberOfHybridisations")){
							numberOfHybridisations=splitline[1];
						}
						
						if (splitline[0].equalsIgnoreCase("NumberOfResultsets")){
							numberOfResultsets=splitline[1];
						}
						if (splitline[0].equalsIgnoreCase("NumberOfBiomaterials")){
							numberOfBiomaterials=splitline[1];
						}
						
						
						inputline=in.readLine();
					}
				}
				inputline = in.readLine();
			}
			System.out.println("Experiment parsed:"+pee.getObj_id());
			return pee;
		}	
			
		private PE_ARRAYS readArrayAnnotation(BufferedReader in )throws IOException{	
			System.out.println("parsing Array Annotation");
			PE_ARRAYS pea = new PE_ARRAYS();
			String inputline = in.readLine();
			String[] splitline;
			while (inputline!=null){
				if (inputline.startsWith("<ArrayCharakteristics")){
					
					inputline=in.readLine();
					while(inputline.startsWith("<")!=true){
						splitline = inputline.split("\t");
					
						
						
						
				        try{
							if (splitline[0].equalsIgnoreCase("OBJ_Identifier")){
								pea.setOBJ_Identifier(splitline[1],array_edb );
								
							}
				        }catch(ArrayIndexOutOfBoundsException e){}
				        
				        try{
							if (splitline[0].equalsIgnoreCase("ObjectName")){
								pea.setOBJ_Name(splitline[1]);
							}
				        }catch(ArrayIndexOutOfBoundsException e){}
					    try{
							if (splitline[0].equalsIgnoreCase("ArrayManufacturer")){
								pea.setArrayManufacturer(splitline[1]);
							}
					    }catch(ArrayIndexOutOfBoundsException e){}
					    
					    try{
							if (splitline[0].equalsIgnoreCase("ArrayTechnology")){
								pea.setArrayTechnology(splitline[1]);
							}
					    }catch(ArrayIndexOutOfBoundsException e){}	
						inputline = in.readLine();
					}
				}
				inputline = in.readLine();
			}
			return pea;
		}	
			
		private Vector<PE_HYBRIDISATIONS> readHybridisationAnnotation(BufferedReader in)throws IOException{
			System.out.println("parsing Hybridisation Annotation");
			Vector<PE_HYBRIDISATIONS> hybs = new Vector<PE_HYBRIDISATIONS>();
			for (int i = 0; i<Integer.parseInt(numberOfHybridisations);i++){
				PE_HYBRIDISATIONS peh= new PE_HYBRIDISATIONS();
				hybs.add(i, peh);
			}
			
			String inputline = in.readLine();
			String[] splitline;
			while (inputline != null){
				
		
				if (inputline.startsWith("<Hybridisation")){
					inputline = in.readLine();
					while (inputline.startsWith("<")!=true){
						splitline = inputline.split("\t");
						
						try{
							if (splitline[0].equalsIgnoreCase("OBJ_Identifier")){
								for(int j=0; j<Integer.parseInt(numberOfHybridisations);j++){
									//hybNames.add(splitline[j+1]);
									hybs.get(j).setOBJ_Identifier(splitline[j+1], edb);
									
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}						
						
						
						try{
							if (splitline[0].equalsIgnoreCase("OBJ_Name")){
								for(int j=0; j<Integer.parseInt(numberOfHybridisations);j++){
									
									hybs.get(j).setOBJ_Name(splitline[j+1]);
									
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("OBJ_Description")){
								for(int j=0; j<Integer.parseInt(numberOfHybridisations);j++){
									hybs.get(j).setOBJ_Description(splitline[j+1]);
									
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("ReplicateDescriptionType")){
								for(int j=0; j<Integer.parseInt(numberOfHybridisations);j++){
									hybs.get(j).setReplicateDescriptionType(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("ReplicateIndex")){
								for(int j=0; j<Integer.parseInt(numberOfHybridisations);j++){
									hybs.get(j).setReplicateIndex(splitline[j+1]);
								}
							}						
						}catch(ArrayIndexOutOfBoundsException e){}
						inputline=in.readLine();
					}
				}
				inputline=in.readLine();
			}	
			
			return hybs;	
		}

		
		private Vector<PE_BIOMATERIALS> readBioMaterialAnnotation(BufferedReader in)throws IOException{
			System.out.println("parsing Biomaterial Annotation");
			Vector<PE_BIOMATERIALS> mats = new Vector<PE_BIOMATERIALS>();
			for (int i = 0; i<Integer.parseInt(numberOfBiomaterials);i++){
				PE_BIOMATERIALS peh= new PE_BIOMATERIALS();
				mats.add(i, peh);
			}
			
			String inputline = in.readLine();
			String[] splitline;
			while (inputline != null){
				
				if (inputline.startsWith("<BioMaterial")){
					inputline = in.readLine();
					
					while (inputline.startsWith("<")!=true){
						splitline = inputline.split("\t");
						
						try{
							if (splitline[0].equalsIgnoreCase("OBJ_Identifier")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setOBJ_Identifier(splitline[j+1], edb);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("OBJ_Name")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setOBJ_Name(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						try{
							if (splitline[0].equalsIgnoreCase("OBJ_Description")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setOBJ_Description(splitline[j+1]);
								}
							}						
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						try{
							if (splitline[0].equalsIgnoreCase("Age")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setAge(splitline[j+1]);
								}
							}						
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("CellType")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setCellType(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("ChromosomalAberrationClassification")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setChromosomalAberrationClassification(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
										
						try{
							if (splitline[0].equalsIgnoreCase("DevelopmentalStage")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setDevelopmentalStage(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						try{
							if (splitline[0].equalsIgnoreCase("DiseaseStaging")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setDiseaseStaging(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						try{
							if (splitline[0].equalsIgnoreCase("DiseaseState")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setDiseaseState(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						try{
							if (splitline[0].equalsIgnoreCase("GrowthCondition")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setGrowthCondition(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("Atmosphere")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setAtmosphere(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("Humidity")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setHumidity(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("Light")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setLight(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						try{
							if (splitline[0].equalsIgnoreCase("Media")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setMedia(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						try{
							if (splitline[0].equalsIgnoreCase("Nutrients")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setNutrients(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("Temperature")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setTemperature(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("Water")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setWater(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("GeneticModification")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setGeneticModification(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("Genotype")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setGenotype(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						try{
							if (splitline[0].equalsIgnoreCase("MatControlType")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setMatControlType(splitline[j+1]);
								}
							}						
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("Organism")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setOrganism(splitline[j+1]);
								}
							}						
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						try{
							if (splitline[0].equalsIgnoreCase("OrganismPart")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setOrganismPart(splitline[j+1]);
								}
							}						
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("Sex")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setSex(splitline[j+1]);
								}
							}						
						}catch(ArrayIndexOutOfBoundsException e){}
						
						
						try{
							if (splitline[0].equalsIgnoreCase("Cultivar")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setCultivar(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("Ecotype")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setEcotype(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
						
						try{
							if (splitline[0].equalsIgnoreCase("TargetedCellType")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setTargetedCellType(splitline[j+1]);
								}
							}	
						}catch(ArrayIndexOutOfBoundsException e){}
						
												
						try{
							if (splitline[0].equalsIgnoreCase("Treatment")){
								for(int j=0; j<Integer.parseInt(numberOfBiomaterials);j++){
									mats.get(j).setTreatment(splitline[j+1]);
								}
							}						
						}catch(ArrayIndexOutOfBoundsException e){}
						

						
		
						
						inputline=in.readLine();
					}
				}
				inputline=in.readLine();
			}	
			
			return mats;	
		}
		
	
		private Vector<PE_RESULTSETS> readResultsetAnnotation(BufferedReader in)throws IOException{
			System.out.println("parsing Resultset Annotation");
			
			Vector<PE_RESULTSETS> rsets = new Vector<PE_RESULTSETS>();
			for (int i = 0; i<Integer.parseInt(numberOfResultsets);i++){
				PE_RESULTSETS per= new PE_RESULTSETS();
				rsets.add(i, per);
			}
			
			
			String inputline = in.readLine();
			String[] splitline;
			//Vector<String> resNames = new Vector<String>(); //to store resultset names for the mapping hashtables
			while (inputline != null){
				
				if (inputline.startsWith("<Resultset")){
					inputline = in.readLine();
					while (inputline.startsWith("<")!=true){
						splitline = inputline.split("\t");
						
						try{
							if (splitline[0].equalsIgnoreCase("OBJ_Identifier")){
								for(int j=0; j<Integer.parseInt(numberOfResultsets);j++){
									rsets.get(j).setOBJ_Identifier(splitline[j+1], edb);
									//System.out.println(splitline[j+1]);
									rsetNames.add(splitline[j+1]);
									//resNames.add(j, splitline[j+1]);
									
								}
							}	
						}catch(ArrayIndexOutOfBoundsException e){
							
						}						
						
						
						try{
							if (splitline[0].equalsIgnoreCase("OBJ_Name")){
								for(int j=0; j<Integer.parseInt(numberOfResultsets);j++){
									rsets.get(j).setOBJ_Name(splitline[j+1]);
									//rsetNames.add(splitline[j+1]);
									//resNames.add(j, splitline[j+1]);
									
								}
							}	
						}catch(ArrayIndexOutOfBoundsException e){
							
						}	
						
						
						
						try{
							if (splitline[0].equalsIgnoreCase("HybridisationName")){
								
								for(int j=0; j<Integer.parseInt(numberOfResultsets);j++){
									
									res2hyb.put(rsetNames.get(j), splitline[j+1]);
									
										
									
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}
							
						try{	
							if (splitline[0].equalsIgnoreCase("BioMaterialName")){
								for(int j=0; j<Integer.parseInt(numberOfResultsets);j++){
									res2mat.put(rsetNames.get(j), splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}	
						
						
						try{	
							if (splitline[0].equalsIgnoreCase("LabelCompound")){
								for(int j=0; j<Integer.parseInt(numberOfResultsets);j++){
									rsets.get(j).setLabelCompound(splitline[j+1]);
								}
							}
						}catch(ArrayIndexOutOfBoundsException e){}	
						
						try{	
							if (splitline[0].equalsIgnoreCase("NormalizationDescriptionType")&& splitline[1]!=null){
								for(int j=0; j<Integer.parseInt(numberOfResultsets);j++){
									rsets.get(j).setNormalizationDescriptionType(splitline[j+1]);
								}
							}						
						}catch(ArrayIndexOutOfBoundsException e){}		
						
						
						try{	
							if (splitline[0].equalsIgnoreCase("ResultSetType")&& splitline[1]!=null){
								for(int j=0; j<Integer.parseInt(numberOfResultsets);j++){
									rsets.get(j).setResultSetType(splitline[j+1]);
								}
							}						
						}catch(ArrayIndexOutOfBoundsException e){}	
						
						
						
						try{	
							if (splitline[0].equalsIgnoreCase("Scale")&& splitline[1]!=null){
								for(int j=0; j<Integer.parseInt(numberOfResultsets);j++){
									rsets.get(j).setScale(splitline[j+1]);
								}
							}						
						}catch(ArrayIndexOutOfBoundsException e){}	
						
						

						
						inputline=in.readLine();
					}
				}
				inputline=in.readLine();
			}	
			
			return	rsets;
		}





		public String getArrayID() {
			return arrayID;
		}





		public void setArrayID(String arrayID) {
			this.arrayID = arrayID;
		}



		/*

		public Vector<String> getHybNames() {
			return hybNames;
		}





		public void setHybNames(Vector<String> hybNames) {
			this.hybNames = hybNames;
		}

		*/



		public Hashtable<String, String> getHybridisationIDs() {
			return hybridisationIDs;
		}





		public void setHybridisationIDs(Hashtable<String, String> hybridisationIDs) {
			this.hybridisationIDs = hybridisationIDs;
		}





		public Hashtable<String, String> getMaterialIDs() {
			return materialIDs;
		}





		public void setMaterialIDs(Hashtable<String, String> materialIDs) {
			this.materialIDs = materialIDs;
		}





		public String getNumberOfBiomaterials() {
			return numberOfBiomaterials;
		}





		public void setNumberOfBiomaterials(String numberOfBiomaterials) {
			this.numberOfBiomaterials = numberOfBiomaterials;
		}





		public String getNumberOfHybridisations() {
			return numberOfHybridisations;
		}





		public void setNumberOfHybridisations(String numberOfHybridisations) {
			this.numberOfHybridisations = numberOfHybridisations;
		}





		public String getNumberOfMaterials() {
			return numberOfBiomaterials;
		}





		public void setNumberOfMaterials(String numberOfMaterials) {
			this.numberOfBiomaterials = numberOfMaterials;
		}





		public String getNumberOfResultsets() {
			return numberOfResultsets;
		}





		public void setNumberOfResultsets(String numberOfResultsets) {
			this.numberOfResultsets = numberOfResultsets;
		}





		public Hashtable<String, String> getRes2hyb() {
			return res2hyb;
		}





		public void setRes2hyb(Hashtable<String, String> res2hyb) {
			this.res2hyb = res2hyb;
		}





		public Hashtable<String, String> getRes2mat() {
			return res2mat;
		}





		public void setRes2mat(Hashtable<String, String> res2mat) {
			this.res2mat = res2mat;
		}





		public Hashtable<String, String> getResultsetIDs() {
			return resultsetIDs;
		}





		public void setResultsetIDs(Hashtable<String, String> resultsetIDs) {
			this.resultsetIDs = resultsetIDs;
		}





		public Vector<String> getRsetNames() {
			return rsetNames;
		}





		public void setRsetNames(Vector<String> rsetNames) {
			this.rsetNames = rsetNames;
		}
	
		

}
