package de.pdwexpress.parser.pLEXdb;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.List;

import de.ipk.pdwexpress.peobjects.PE_ARRAYS;
import de.ipk.pdwexpress.peobjects.PE_BIOMATERIALS;
import de.ipk.pdwexpress.peobjects.PE_BIOSEQUENCES;
import de.ipk.pdwexpress.peobjects.PE_EXPERIMENTS;
import de.ipk.pdwexpress.peobjects.PE_ExternDBEnum;
import de.ipk.pdwexpress.peobjects.PE_HYBRIDISATIONS;
import de.ipk.pdwexpress.peobjects.PE_RESULTSETS;
import de.ipk.pdwexpress.upload.AbstractPEObject;
import de.ipk.pdwexpress.upload.ExternDBInterface;
import de.ipk.pdwexpress.upload.ImportHandler;
import de.pdwexpress.parser.common.MatrixParser;


public class Import {
	
	ImportHandler importHandler;
	Connection con;
	
	String dataFileName;
	String resultsetFileName;
	String materialFileName;
	String hybridisationFileName;
	String experimentFileName;
	String arrayForeignKey;
	
	
	
	public Import(ImportHandler importHandler, Connection con)throws SQLException {
		super();
		// TODO Auto-generated constructor stub
		this.importHandler = importHandler;
		this.con = con;
		
		PE_ARRAYS myArray = new PE_ARRAYS();
		myArray.setArrayManufacturer("Affymetrix");
		myArray.setArrayTechnology("Affymetrix");
		myArray.setOBJ_Identifier("A-AFFY-31",PE_ExternDBEnum.ArrayExpress);
		myArray.setOBJ_Name("Affymetrix GeneChip Barley Genome [Barley1]");
		arrayForeignKey=myArray.saveObject();
		
	}



	public String insertExperiment(String obID)throws IOException,NoSuchMethodException, ClassNotFoundException, SQLException, FileNotFoundException{
	
		
		String expObjKey=new String();
		try{
		
			
			MatrixParser<PE_EXPERIMENTS> ap = new MatrixParser<PE_EXPERIMENTS>(new PE_EXPERIMENTS());
			List<PE_EXPERIMENTS> myExp = ap.parseFile(new File(experimentFileName));
		
			
		
			for (PE_EXPERIMENTS ex:myExp ){
				ex.setOBJ_Identifier(obID, PE_ExternDBEnum.PLEXdb);
				ex.saveObject();
				expObjKey=ex.getObj_id();
			}
			//con.commit();
		}catch (SQLException e){
			con.rollback();
			throw e;
		}
		
		return expObjKey;
	}

	
	
	public Hashtable<String, String> insertHybridisations(String expID)
		throws ClassNotFoundException, SQLException, FileNotFoundException, IOException, NoSuchMethodException{
		
		Hashtable<String, String> hybIDs= new Hashtable<String, String>();
		
		//Class.forName("oracle.jdbc.driver.OracleDriver");
		//Connection con = DriverManager.getConnection("jdbc:oracle:thin:@pgrc-16.ipk-gatersleben.de:1521:pdw","pdw_express_stage","");
		
		try{
			//ImportHandler importHandler = new ImportHandler(con);
			AbstractPEObject.setImportHandler(importHandler);
		
			MatrixParser<PE_HYBRIDISATIONS> ap = new MatrixParser<PE_HYBRIDISATIONS>(new PE_HYBRIDISATIONS());
			List<PE_HYBRIDISATIONS> myExp = ap.parseFile(new File(hybridisationFileName));
		

			
		
			for (PE_HYBRIDISATIONS ex:myExp ){
				ex.setFK_HYB_EXP_ID(expID);
				ex.setFK_HYB_ARY_ID(arrayForeignKey);
				ex.saveObject();
				hybIDs.put(ex.getOBJ_Name(),ex.getObj_id());
			}
			//con.commit();
		}catch(SQLException e){
			con.rollback();
			throw e;
		}
		//finally{con.close();}
		return hybIDs;
	}
		
	
	
	public Hashtable<String, String> insertMaterials()
		throws ClassNotFoundException, SQLException, FileNotFoundException, IOException, NoSuchMethodException{
	
			Hashtable<String, String> matIDs= new Hashtable<String, String>();
		
			//Class.forName("oracle.jdbc.driver.OracleDriver");
			//Connection con = DriverManager.getConnection("jdbc:oracle:thin:@pgrc-16.ipk-gatersleben.de:1521:pdw","pdw_express_stage","");
		try{
			//ImportHandler importHandler = new ImportHandler(con);
			AbstractPEObject.setImportHandler(importHandler);
		
			MatrixParser<PE_BIOMATERIALS> ap = new MatrixParser<PE_BIOMATERIALS>(new PE_BIOMATERIALS());
			List<PE_BIOMATERIALS> myExp = ap.parseFile(new File(materialFileName));
		
			for (PE_BIOMATERIALS ex:myExp ){
				ex.saveObject();
				matIDs.put(ex.getOBJ_Name(),ex.getObj_id());
			}
			//con.commit();	
		}catch (SQLException e){
			con.rollback();
			throw e;
		}
		//finally {con.close();}	
		
		return matIDs;
	}
	
	
	
	
	
	
	public Hashtable<String, String> insertResultsets(Hashtable<String, String> hybIDs, Hashtable<String, String> matIDs)
	throws ClassNotFoundException, SQLException, FileNotFoundException, IOException, NoSuchMethodException{
	
			Hashtable<String, String> resIDs= new Hashtable<String, String>();
	
			//Class.forName("oracle.jdbc.driver.OracleDriver");
			//Connection con = DriverManager.getConnection("jdbc:oracle:thin:@pgrc-16.ipk-gatersleben.de:1521:pdw","pdw_express_stage","");
	    try{
	    	//ImportHandler importHandler = new ImportHandler(con);
	    	AbstractPEObject.setImportHandler(importHandler);
	
	
	    	MatrixParser<PE_RESULTSETS> ap = new MatrixParser<PE_RESULTSETS>(new PE_RESULTSETS());
	    	List<PE_RESULTSETS> myExp = ap.parseFile(new File(resultsetFileName));
	
	
	
	    	for (PE_RESULTSETS ex:myExp ){
	    		ex.setFK_RST_HYB_ID(hybIDs.get(ex.getOBJ_Name()));
	    		ex.setFK_RST_MAT_ID(matIDs.get(ex.getOBJ_Name()));
	    		ex.saveObject();
	    		resIDs.put(ex.getOBJ_Name(),ex.getObj_id());
	    		//con.commit();
	    	}
	    }catch (SQLException e){
	    	con.rollback();
	    	throw e;}
	    //finally{con.close();}
	
	    return resIDs;
	}	
	
	
	
	
	
	
	
	public boolean insertResult_Data( Hashtable<String,String> ResultsetIDs )
	throws SQLException, IOException, ClassNotFoundException{
		
//		 create connection
		
		//Class.forName("oracle.jdbc.driver.OracleDriver");
		//Connection con = DriverManager.getConnection("jdbc:oracle:thin:@pgrc-16.ipk-gatersleben.de:1521:pdw","pdw_express_stage","");
		
		
//		 Assign Connection to import API
//		 Auto comit will be set th false !
		//ImportHandler importHandler = new ImportHandler(con);
		AbstractPEObject.setImportHandler(importHandler);
		
//		 read in the data file		
		
		BufferedReader in = new BufferedReader(new FileReader(dataFileName));
		String[] HybNames = in.readLine().split("\t");
		
		try{		
		
		in.readLine();
		String inputline=in.readLine();
		String statement = "insert into RESULT_DATA_T (RSD_OBJ_ID_RST, RSD_OBJ_ID, RSD_Value) " +
		"values (? , ? ,  ?)";
		PreparedStatement pstmt = con.prepareStatement(statement);
		
		
		
		
		while(inputline != null){
			String[] values= inputline.split("\t");
			
			//insert the Biosequence and remember the Primary KEy
			PE_BIOSEQUENCES bioS = new PE_BIOSEQUENCES();
			bioS.setOBJ_Identifier(values[0],PE_ExternDBEnum.Affymetrix );
			bioS.saveObject();
			String GeneKey = bioS.getObj_id();
			

			for(int i=1;i<values.length;i++){
				String RSetKey=ResultsetIDs.get(HybNames[i]);
				pstmt.setInt(1, Integer.parseInt(RSetKey));
				pstmt.setInt(2, Integer.parseInt(GeneKey));
				pstmt.setDouble(3, Double.parseDouble(values[i]));
				pstmt.execute();
			}
			
			
			
			
			inputline=in.readLine();
		}
		
		
		pstmt.close();
		in.close();
		//con.commit();
		
		}catch (SQLException e){
			con.rollback();
			throw e;
		}
		//finally{con.close();}
		return true;
	}



	public String getDataFileName() {
		return dataFileName;
	}



	public void setDataFileName(String dataFileName) {
		this.dataFileName = dataFileName;
	}



	public String getExperimentFileName() {
		return experimentFileName;
	}



	public void setExperimentFileName(String experimentFileName) {
		this.experimentFileName = experimentFileName;
	}



	public String getHybridisationFileName() {
		return hybridisationFileName;
	}



	public void setHybridisationFileName(String hybridisationFileName) {
		this.hybridisationFileName = hybridisationFileName;
	}



	public String getMaterialFileName() {
		return materialFileName;
	}



	public void setMaterialFileName(String materialFileName) {
		this.materialFileName = materialFileName;
	}



	public String getResultsetFileName() {
		return resultsetFileName;
	}



	public void setResultsetFileName(String resultsetFileName) {
		this.resultsetFileName = resultsetFileName;
	}
	
	
	
	
	
	
	
	
	
}